/*
 * Simple program to generate HTML reports based on Red Hill's Simian
 * analyzer output.
 *
 * Copyright (c) 2004 Aaron Grothe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXLINE 4096

// tokens used to interpert Simian output file.
// May need to be changed if Simian changes output file format

#define BETWEEN_TOKEN " Between"
#define FOUND_TOKEN   "Found"
#define BLOCKS_TOKEN  "blocks"
#define LOADING_TOKEN "Loading"

static FILE *in_fp;
static FILE *out_fp;

static int total_dup_lines;
static int total_dup_blocks;
static int total_dup_files;
static size_t dir_offset;

static void found_line(const char *buffer)
{
    int duplicate_lines = 0;

    if (strstr(buffer, BLOCKS_TOKEN))	// check for total line
    {
	(void) sscanf(buffer,
		      "Found %d duplicate lines in %d blocks in %d files\n",
		      &total_dup_lines, &total_dup_blocks,
		      &total_dup_files);
    } else {			// begins a new section
	(void) sscanf(buffer, "Found %d duplicate lines ",
		      &duplicate_lines);
#ifdef DEBUG
	printf("duplicate_lines = %d\n", duplicate_lines);
#endif
	(void) fprintf(out_fp,
		       "<br>Found %d duplicate lines in the following files:<br>\n",
		       duplicate_lines);

    }

}

static void cut_file(const char *filename, const int start_line,
		     const int end_line)
{
    FILE *in_fp2 = NULL;
    FILE *out_fp2 = NULL;
    char outfile_name[MAXLINE];
    char *ptr = NULL;
    char line[MAXLINE];
    int i;
    size_t j;

#ifdef DEBUG
    fprintf(stderr, "cut_file::filename = %s\n", filename);
#endif

    in_fp2 = fopen(filename, "r");
    if (in_fp2 == NULL) {
	(void) fprintf(stderr, "unable to open %s\n", filename);
	exit(EXIT_FAILURE);
    }

    strcpy(outfile_name, filename);
    for (ptr = outfile_name; *ptr != '\0'; ptr++) {
	if (*ptr == '/') {
	    *ptr = '_';
	}
	if (*ptr == '.') {
	    *ptr = '_';
	}
    }

    (void) sprintf(outfile_name, "%s_%d_%d.html", outfile_name,
		   start_line, end_line);

    out_fp2 = fopen(outfile_name, "w");
    if (out_fp2 == NULL) {
	(void) fprintf(stderr, "unable to open %s\n", filename);
	(void) fclose(in_fp2);
	exit(EXIT_FAILURE);
    }
    // put header on file

    (void) fprintf(out_fp2, "<html>\n"
		   "<head>\n"
		   "<meta content=\"text/html; charset=ISO-8859-1\"\n"
		   "http-equiv=\"content-type\">\n"
		   "<title></title>\n" "</head>\n" "<body>\n");

    (void) fprintf(out_fp2, "<hr style=\"width: 100%%; height: 2px;\">\n");
    (void) fprintf(out_fp2, "<b>File:</b> %s<br>\n", filename);
    (void) fprintf(out_fp2, "<b>Lines:</b> %d - %d\n", start_line,
		   end_line);
    (void) fprintf(out_fp2, "<hr style=\"width: 100%%; height: 2px;\">\n");

    i = 1;

    while (fgets(line, MAXLINE, in_fp2) != NULL) {

	if ((i >= start_line) && (i < end_line)) {
	    (void) fprintf(out_fp2,
			   "<span style=\"font-family: courier new,courier,monospace;\">\n");
	    (void) fprintf(out_fp2, "%6.6d:&nbsp;", i);
	    for (j = 0; j < strlen(line); j++) {
		switch (line[j]) {
		case ' ':
		    (void) fprintf(out_fp2, "&nbsp;");
		    break;
		case '\t':
		    (void) fprintf(out_fp2, "&nbsp;&nbsp;&nbsp;&nbsp;");
		    break;
		default:
		    (void) fputc(line[j], out_fp2);
		    break;
		}
	    }
	    (void) fprintf(out_fp2, "</span>\n");
	    (void) fprintf(out_fp2, "<br>\n");
	}
	i++;
    }

    (void) fclose(in_fp2);


    // insert sourceforge logo
    (void) fprintf(out_fp2, "<br><a href=\"http://sourceforge.net\"><img "
		   "src=\"http://sourceforge.net/sflogo.php?group_id=linuxcpd&amp;type=7\""
		   "width=\"88\" height=\"31\" border=\"0\" alt=\"SourceForge.net Logo\"></a>\n");

    // print closing line
    //
    (void) fprintf(out_fp2, "<br>\n" "</body>\n" "</html>\n");

    (void) fclose(out_fp2);

#ifdef DEBUG
    printf("outfile_name = %s\n", outfile_name);
#endif


    (void) fprintf(out_fp,
		   "<div style=\"margin-left: 40px;\"><a href=%s>lines %d-%d %s<br></a></div>\n",
		   outfile_name, start_line, end_line, filename);

}

static void between_line(const char *buffer)
{

    int start_line = 0, end_line = 0;
    char filename[MAXLINE];

    (void) sscanf(buffer, " Between lines %d and %d in %s\n",
		  &start_line, &end_line, filename);

#ifdef DEBUG
    printf("start_line = %d\n", start_line);
    printf("end_line = %d\n", end_line);
    printf("filename = %s\n", filename + dir_offset);
#endif
    cut_file(filename + dir_offset, start_line, end_line);

}

static void loading_line(const char *buffer)
{
    char start[MAXLINE];

    (void) sscanf(buffer, "Loading (recursively) *.c from %s\n", start);
    dir_offset = strlen(start) + 1;	// set to base dir +1 for trailing slash

#ifdef DEBUG
    printf("start = %s\n", start);
    printf("dir_offset = %d\n", dir_offset);
#endif


}

int main(int argc, char *argv[])
{

    char buffer[MAXLINE];

    if (argc != 3) {
	(void) fprintf(stderr, "usage: %s inputfile outputfile\n",
		       argv[0]);
	exit(EXIT_FAILURE);
    }

    in_fp = fopen(argv[1], "r");
    if (in_fp == NULL) {
	(void) fprintf(stderr, "Unable to open input file %s\n", argv[1]);
	exit(EXIT_FAILURE);
    }

    out_fp = fopen(argv[2], "w");
    if (out_fp == NULL) {
	(void) fprintf(stderr, "Unable to open output file %s\n", argv[2]);
	(void) fclose(in_fp);
	exit(EXIT_FAILURE);
    }

    (void) fprintf(out_fp, "<html>\n");
    (void) fprintf(out_fp, "<head>\n");
    (void) fprintf(out_fp,
		   "<meta content=\"text/html; charset=ISO-8859-1\"\n");
    (void) fprintf(out_fp, "http-equiv=\"content-type\">\n");
    (void) fprintf(out_fp, "<title>LinuxCPD Result</title>\n");
    (void) fprintf(out_fp, "</head>\n");
    (void) fprintf(out_fp, "<body>\n");


// print info

    (void) fprintf(out_fp,
		   "<div style=\"text-align: center;\"><h1><span style=\"font-weight: bold;\">Linux CPD Report</span></h1><br>"
		   "</div>\n" "<br>\n");
    while (fgets(buffer, MAXLINE, in_fp) != NULL) {
	if (strncmp(buffer, BETWEEN_TOKEN, strlen(BETWEEN_TOKEN)) == 0) {
	    between_line(buffer);
	} else {
	    if (strncmp(buffer, FOUND_TOKEN, strlen(FOUND_TOKEN)) == 0) {
		found_line(buffer);
	    } else {
		if (strncmp(buffer, LOADING_TOKEN, strlen(LOADING_TOKEN))
		    == 0) {
		    loading_line(buffer);
		}
		// should be a throwaway
	    }
	}
    }
    (void) fclose(in_fp);

    // print html table for end of report

    (void) fprintf(out_fp, "<br><br>\n");
    (void) fprintf(out_fp,
		   "<table style=\"text-align: left; width: 100%%;\" border=\"1\" cellspacing=\"2\"\n");
    (void) fprintf(out_fp, "	cellpadding=\"2\">\n");
    (void) fprintf(out_fp, "	<tbody>\n");
    (void) fprintf(out_fp, "	<tr>\n");
    (void) fprintf(out_fp,
		   "	<td style=\"vertical-align: top;\"><b>Number of Duplicate Blocks<br></b>\n");
    (void) fprintf(out_fp, "	</td>\n");
    (void) fprintf(out_fp,
		   "	<td style=\"vertical-align: top;\">%d</td>\n",
		   total_dup_blocks);
    (void) fprintf(out_fp, "	</tr>\n");
    (void) fprintf(out_fp, "	<tr>\n");
    (void) fprintf(out_fp,
		   "	<td style=\"vertical-align: top;\"><b>Number of affected Files</b><br>\n");
    (void) fprintf(out_fp, "	</td>\n");
    (void) fprintf(out_fp,
		   "	<td style=\"vertical-align: top;\">%d<br>\n",
		   total_dup_files);
    (void) fprintf(out_fp, "	</td>\n");
    (void) fprintf(out_fp, "	</tr>\n");
    (void) fprintf(out_fp, "	<tr>\n");
    (void) fprintf(out_fp,
		   "	<td style=\"vertical-align: top;\"><b>Total Number of Duplicated Lines</b><br>\n");
    (void) fprintf(out_fp, "	</td>\n");
    (void) fprintf(out_fp,
		   "	<td style=\"vertical-align: top;\">%d<br>\n",
		   total_dup_lines);
    (void) fprintf(out_fp, "	</td>\n");
    (void) fprintf(out_fp, "	</tr>\n");
    (void) fprintf(out_fp, "	</tbody>\n");
    (void) fprintf(out_fp, "	</table>\n");

    // insert sourceforge logo
    (void) fprintf(out_fp, "<br><a href=\"http://sourceforge.net\"><img "
		   "src=\"http://sourceforge.net/sflogo.php?group_id=linuxcpd&amp;type=7\""
		   "width=\"88\" height=\"31\" border=\"0\" alt=\"SourceForge.net Logo\"></a>\n");

    (void) fprintf(out_fp, "</body>\n</html>\n");

    (void) fclose(out_fp);

#ifdef DEBUG
    printf("total_dup_lines = %d\n", total_dup_lines);
    printf("total_dup_blocks = %d\n", total_dup_blocks);
    printf("total_dup_files = %d\n", total_dup_files);
#endif

    return 0;

}
